local function bird_mod_internal(str, pn)
	local ps= GAMESTATE:GetPlayerState(pn)
	local pmods= ps:GetPlayerOptionsString('ModsLevel_Song')
	ps:SetPlayerOptions('ModsLevel_Song', pmods .. ', ' .. str)
	--GAMESTATE:ApplyGameCommand('mod,'..str, pn)
end

local function bird_mod(str)
	for i=1,2 do
		bird_mod_internal(str, 'PlayerNumber_P' .. i)
	end
end

function bird_saveFileExists()
	local path=GAMESTATE:GetCurrentSong():GetSongDir();
	if FILEMAN:DoesFileExist(path.."lua/savefile.lua") then
		return true
	end
	return false
end

function bird_writeSaveFile()
	local file = RageFileUtil.CreateRageFile()
	local path=GAMESTATE:GetCurrentSong():GetSongDir();
	file:Open( path.."lua/savefile.lua", 2 ) --2 = write
	file:PutLine("bird_played = "..bird_played)
	file:PutLine("--")
	file:PutLine("ut_LOVE = "..ut_LOVE)
	file:Close();
end

if bird_saveFileExists() then
	--SCREENMAN:SystemMessage('Save File exists, loading');
	LoadActor("savefile")
else
	--SCREENMAN:SystemMessage('Save File does not exist, creating new');
	bird_played = 0
	if not ut_LOVE then ut_LOVE = 0 end
	bird_writeSaveFile()
end

if not ut_LOVE then ut_LOVE = 0 end

local function bird_init()
	--sprite pool for the pollen effects
	bird_pollentable = {}
	--pointer to a pollen object index
	bird_pollenptr = 1;
	
	--this function spawns some pollen at a given location
	function bird_spawnpollen(x,y)
		local a = bird_pollentable[bird_pollenptr]
		
		--perform pollen actions
		if a then
			a:finishtweening() --reset
			a:visible(true)
			--a:blend('add');
			a:zoom(math.random(15,45)/100);
			a:x(x)
			a:y(y)
			a:diffusealpha(4)
			a:decelerate(math.random(250,300)/100);
			a:addy(-math.random(100,200))
			a:addx(math.random(0,100)-50);
			a:diffusealpha(0);
			a:queuecommand('Hide'); --hide when we're done, so we don't waste memory
		end
		
		--increment pollen pointer to get the next piece of pollen
		bird_pollenptr = bird_pollenptr+1;
		--but if we reach the end of the list, start again from the first one
		if bird_pollenptr > table.getn(bird_pollentable) then
			bird_pollenptr = 1;
		end
	end
	
	bird_spawntimer = 1;

	--which direction are we travelling today
	bird_dir = 1; -- -1 = left, 1 = right
	
	--if we have played an even number of times, we're travelling left, otherwise right
	if (bird_played/2)-math.floor(bird_played/2) == 0.5 then
		bird_dir = 1
	else
		bird_dir = -1;
	end
	
	bird_nobodycame = false;
	
	--testing purposes
	--ut_LOVE = 100;
	
	--increment play count
	if ut_LOVE < 5 then
		bird_played = bird_played+1;
	else
		bird_nobodycame = true;
		ut_LOVE = 0;
	end
	
	bird_writeSaveFile();
	
	Trace('Played: '..bird_played);
	Trace('LOVE: '..ut_LOVE);

	checked = false;
	
	mod_firstSeenBeat = GAMESTATE:GetSongBeat()

	--actions table - a list of actions to perform at various times within the song
	curaction = 1;
	--{beat,action,persist}
	actions = {
		--run this function at beat 0
		{0.1,function()
		
			--since we shrink the playfield, we want the notes to fade in rather than pop in, so 'sudden' is used
			--'120 suddenoffset' controls the fade-in point on the screen,
			--bigger numbers = more visible (lower fade-in point), smaller/negative numbers = less visible
			GAMESTATE:ApplyGameCommand('mod,*10000 120 suddenoffset, *10 sudden');
			
			--scrunch the playfields a little
			GAMESTATE:ApplyGameCommand('mod,*2 20 reverse, *1 10 flip');
			
			--max bg brightness
			GAMESTATE:ApplyGameCommand('mod,*1000 -1000 cover');
			
			--check if player 1 exists
			if P1 then
				
				--move and resize play area
				P1:linear(0.1)
				if bird_dir == -1 then
					P1:x(SCREEN_WIDTH*0.85)
				else
					P1:x(SCREEN_WIDTH*0.15)
				end
				P1:y(SCREEN_CENTER_Y-50)
				P1:zoom(0.55)
				
			end
			
			--check if player 2 exists
			if P2 then
				
				--move and resize play area
				P2:linear(0.1)
				if bird_dir == -1 then
					P2:x(SCREEN_WIDTH*0.85)
				else
					P2:x(SCREEN_WIDTH*0.15)
				end
				P2:y(SCREEN_CENTER_Y-50)
				P2:zoom(0.55)
			end
			
			--moving judgments and combo is a bit different in SM5
			if P1 and P2 then
				local jud1 = P1:GetChild('Judgment')
				local com1 = P1:GetChild('Combo')
				local jud2 = P2:GetChild('Judgment')
				local com2 = P2:GetChild('Combo')
				
				--retarded
				jud1:bob()
				jud1:effectmagnitude(-160,0,0)
				jud1:effectperiod(1000000)
				jud1:effectoffset(250000)
				com1:bob()
				com1:effectmagnitude(-160,0,0)
				com1:effectperiod(1000000)
				com1:effectoffset(250000)
				jud2:bob()
				jud2:effectmagnitude(160,0,0)
				jud2:effectperiod(1000000)
				jud2:effectoffset(250000)
				com2:bob()
				com2:effectmagnitude(160,0,0)
				com2:effectperiod(1000000)
				com2:effectoffset(250000)
			end
			
			--place the bird
			char_bird:visible(true);
			if bird_dir == -1 then
				char_bird:x(SCREEN_WIDTH*0.72);
			else
				char_bird:x(SCREEN_WIDTH*0.28);
			end
			char_bird:y(SCREEN_CENTER_Y+20);
			
		end,true},
		
		--move the bird over to the playfield
		{4,function()
			char_bird:playcommand('Fly');
			char_bird:linear(1.2);
			if bird_dir == -1 then
				char_bird:x(SCREEN_WIDTH*0.85);
			else
				char_bird:x(SCREEN_WIDTH*0.15);
			end
			char_bird:y(SCREEN_CENTER_Y-90);
		end},
		
		{10,function()
			char_bird:linear(1.06);
			char_bird:addy(-40);
		end},
		{12,function()
			
			--vibrate the bird just a little
			char_bird:vibrate();
			char_bird:effectmagnitude(1,1,0);
		
			--get both players at once, that works too
			for pn=1,2 do
				local a = _G['P'..pn] --store P1/P2 in 'a'
				if a then --don't forget to check if 'a' exists tho
					a:linear(0.5);
					a:addy(-16);
					a:vibrate();
					a:effectmagnitude(2,2,0);
				end
			end
		end},
		
		{16,function()
			
			--bird travel across the disproportionately small gap
			char_bird:linear(32*(60/170)) --1 beat = 60/bpm in seconds, carry takes 32 beats
			if bird_dir == -1 then
				char_bird:x(SCREEN_WIDTH*0.15)
			else
				char_bird:x(SCREEN_WIDTH*0.85)
			end
			
			--carry players across the disproportionately small gap
			for pn=1,2 do
				local a = _G['P'..pn] --store P1/P2 in 'a'
				if a then --don't forget to check if 'a' exists tho
					a:linear(32*(60/170));
					if bird_dir == -1 then
						a:x(SCREEN_WIDTH*0.15);
					else
						a:x(SCREEN_WIDTH*0.85);
					end
				end
			end
			
		end},
		
		{48,function()
			char_bird:linear(1.06);
			char_bird:addy(40);
		end},
		{48,function()
		
			--get both players at once, that works too
			for pn=1,2 do
				local a = _G['P'..pn]
				if a then
					a:linear(0.5);
					a:addy(16);
				end
			end
		end},
		
		{50,function()
		
			char_bird:stopeffect(); --stop vibration
		
			--get both players at once, that works too
			for pn=1,2 do
				local a = _G['P'..pn]
				if a then 
					a:stopeffect(); --stop vibration
				end
			end
		end},
		
		{52,function()
			char_bird:linear(0.52);
			char_bird:y(SCREEN_CENTER_Y-40);
			char_bird:linear(1.05);
			if bird_dir == -1 then
				char_bird:x(SCREEN_WIDTH*0.3);
			else
				char_bird:x(SCREEN_WIDTH*0.7);
			end
			char_bird:linear(0.52);
			char_bird:y(SCREEN_CENTER_Y+20);
		end},
		
		{58,function()
			char_bird:playcommand('Idle');
		end},
		
	}
	
	
	
	----super secret lolol
	curloveaction = 1;
	--{beat,action,persist}
	loveactions = {
		{0.1,function()
		
			--since we shrink the playfield, we want the notes to fade in rather than pop in, so 'sudden' is used
			--'120 suddenoffset' controls the fade-in point on the screen,
			--bigger numbers = more visible (lower fade-in point), smaller/negative numbers = less visible
			GAMESTATE:ApplyGameCommand('mod,*10000 120 suddenoffset, *10 sudden');
			
			--scrunch the playfields a little
			GAMESTATE:ApplyGameCommand('mod,*2 20 reverse, *1 10 flip');
			
			--max bg brightness
			GAMESTATE:ApplyGameCommand('mod,*1000 -1000 cover');
			
			--check if player 1 exists
			if P1 then
				
				--move and resize play area
				P1:linear(0.1)
				if bird_dir == -1 then
					P1:x(SCREEN_WIDTH*0.85)
				else
					P1:x(SCREEN_WIDTH*0.15)
				end
				P1:y(SCREEN_CENTER_Y-50)
				P1:zoom(0.55)
				
				--move the judgments and combo so they aren't on top of the playfield
				local jud1 = P1:GetChild('Judgment');
				local com1 = P1:GetChild('Combo');
				
				jud1:linear(0.1);
				jud1:x(-160);
				com1:linear(0.1);
				com1:x(-160);
				
			end
			
			--check if player 2 exists
			if P2 then
				
				--move and resize play area
				P2:linear(0.1)
				if bird_dir == -1 then
					P2:x(SCREEN_WIDTH*0.85)
				else
					P2:x(SCREEN_WIDTH*0.15)
				end
				P2:y(SCREEN_CENTER_Y-50)
				P2:zoom(0.55)
				
				--move the judgments and combo so they aren't on top of the playfield
				local jud2 = P2:GetChild('Judgment');
				local com2 = P2:GetChild('Combo');
				
				jud2:linear(0.1);
				jud2:x(160);
				com2:linear(0.1);
				com2:x(160);
			end
			
			--no bird lol
			
		end,true},
		{8,function()
			MESSAGEMAN:Broadcast('Nobody');
		end},
	}
	
	--sorting algorithm used in the next block
	function modtable_compare(a,b)
		return a[1] < b[1]
	end
	
	--sort the actions table in order of when they happen in the song
	if table.getn(actions) > 1 then
		table.sort(actions, modtable_compare)
	end
	
	--sort the loveactions table in order of when they happen in the song
	if table.getn(loveactions) > 1 then
		table.sort(loveactions, modtable_compare)
	end
	
end

local function bird_update(self, delta)

	if GAMESTATE:GetSongBeat()>0 and not checked then
		
		P1 = SCREENMAN:GetTopScreen():GetChild('PlayerP1');
		P2 = SCREENMAN:GetTopScreen():GetChild('PlayerP2');
		
		checked = true;
	end
	
	--shortcut for the current song beat
	local beat = GAMESTATE:GetSongBeat()
	
	--spawn pollen
	if beat > bird_spawntimer then
		bird_spawnpollen(math.random(10,SCREEN_WIDTH-10),math.random(80,SCREEN_HEIGHT-60));
		bird_spawntimer = bird_spawntimer+0.2
	end
	
	--check each frame to see if an action from the actions table should be executed
	if beat > mod_firstSeenBeat+0.1 then
		
		--if the current action should be played, play it then move to the next action
		--the 3rd flag [3], 'persists', is less important so I won't bother explaining it lol
		--it's an editor thing
		
		--oh, and if your LOVE is too high, don't do anything! nobody came!
		if not bird_nobodycame then
			while curaction<=table.getn(actions) and GAMESTATE:GetSongBeat()>=actions[curaction][1] do
				if actions[curaction][3] or GAMESTATE:GetSongBeat() < actions[curaction][1]+2 then
					actions[curaction][2]() --run action
				end
				curaction = curaction+1; --goto next action
			end
		else
			while curloveaction<=table.getn(loveactions) and GAMESTATE:GetSongBeat()>=loveactions[curloveaction][1] do
				if loveactions[curloveaction][3] or GAMESTATE:GetSongBeat() < loveactions[curloveaction][1]+2 then
					loveactions[curloveaction][2]() --run loveaction
				end
				curloveaction = curloveaction+1; --goto next loveaction
			end
		end
		
	end
	
end

bird_pollenframe = Def.ActorFrame{}

--set up particle system
for i=1,512 do
	bird_pollenframe[#bird_pollenframe+1] = LoadActor("pollen")..{
		OnCommand=cmd(visible,false; sleep,0.02; queuecommand,"SetMe";),
		SetMeCommand=function(self) table.insert(bird_pollentable,self); end,
		HideCommand=cmd(visible,false),
	}
end

return Def.ActorFrame{
	OnCommand= function(self)
							 bird_init()
							 self:SetUpdateFunction(bird_update)
							 
						 end,
	Def.Quad{
		Name= "I may be sleeping, but I preserve the world.",
		InitCommand= cmd(visible,false),
		OnCommand= cmd(sleep,1000),
	},
	
	--The star of the show
	--we play the "Idle" command on the "char_bird" object we defined to hide the flying sprite and show the idle sprite
	--conversely, the "Fly" command hides the idle sprite
	Def.ActorFrame{
		InitCommand=function(self) char_bird = self end,
		OnCommand=cmd(visible,false),
		LoadActor("idle.lua")..{
			OnCommand=cmd(diffusealpha,1),
			FlyCommand=cmd(diffusealpha,0),
			IdleCommand=cmd(diffusealpha,1),
		},
		LoadActor("fly.lua")..{
			OnCommand=cmd(diffusealpha,0),
			FlyCommand=cmd(diffusealpha,1),
			IdleCommand=cmd(diffusealpha,0),
		},
	},
	
	bird_pollenframe,
	
	LoadActor("nobody")..{
		OnCommand=cmd(cropright,1;x,SCREEN_CENTER_X;y,SCREEN_CENTER_Y-50;diffuse,0,0,0,1;),
		NobodyMessageCommand=cmd(linear,1;cropright,0),
	}
	
}